from django.db import models
from django.contrib.auth.models import User
import re

class Base(models.Model):
    """Abstract base class for all model classes."""
    name = models.CharField(max_length=200)
    def __unicode__(self):
        return self.name
    class Meta:
        abstract = True


class Collection(Base):
    """A collection is a grouping of Volumes.

    Usually a library will have a number of different collections,
    e.g. "Southeast Asia Collection", "Correspondence of John Irving", etc.
    """
    source = models.CharField(max_length=200)
    publication_date = models.DateTimeField('date published')
    curator = models.ForeignKey(User)


class Storage(Base):
    """Storage is a reference to the physical media where the work is located.

    Volumes can be found by using the Storage mount_point as the root.
    """
    mount_point = models.CharField(max_length=100)
    class Meta(Base.Meta):
        ordering = ['name']
        verbose_name_plural = 'storage'

class Set(Base):
    pass


class Condition(Base):
    pass


class Person(Base):
    class Meta(Base.Meta):
        ordering = ['id']


class PrintType(Base):
    pass


class Language(Base):
    pass


class Subject(Base):
    pass


class MediaType(Base):
    pass


def make_pairs(a, b, default='-'):
    ret = filter(None, (a, b))
    return ret or default

class Metadata(models.Model):
    """Metadata for a volume."""
    def __unicode__(self):
        return self.nlvnpf_num
    condition = models.ForeignKey(Condition)
    # contributor = models.CharField(max_length=20)
    coverage = models.CharField(max_length=50)
    creator_hn = models.CharField(max_length=100)
    creator_qn = models.CharField(max_length=500)
    date_dl = models.CharField(max_length=50)
    date_hn = models.CharField(max_length=50)
    date_image = models.CharField(max_length=10)
    date_qn = models.CharField(max_length=200)
    description = models.CharField(max_length=3500)
    description_en = models.CharField(max_length=3500)
    digitizer = models.ForeignKey(Person)
    print_type = models.ForeignKey(PrintType)
    formats = models.CharField(max_length=20)
    height = models.CharField(max_length=10)
    legacy_id = models.IntegerField()
    # illustration = models.CharField(max_length=20)
    language = models.ForeignKey(Language)
    nlv_num = models.CharField(max_length=20)
    nlvnpf_num = models.CharField(max_length=20)
    notes = models.CharField(max_length=1500)
    notes_en = models.CharField(max_length=1500)
    pages = models.IntegerField()
    place_hn = models.CharField(max_length=100)
    place_qn = models.CharField(max_length=100)
    publisher_hn = models.CharField(max_length=100)
    publisher_qn = models.CharField(max_length=100)
    relation = models.CharField(max_length=50)
    rights = models.CharField(max_length=100)
    source = models.CharField(max_length=100)
    subject = models.ForeignKey(Subject)
    title_alt_hn = models.CharField(max_length=200)
    title_alt_qn = models.CharField(max_length=200)
    title_hn = models.CharField(max_length=200)
    title_qn = models.CharField(max_length=200)
    media_type = models.ForeignKey(MediaType)
    width = models.CharField(max_length=10)

    def creators(self):
        return make_pairs(self.creator_hn, self.creator_qn)

    def dates(self):
        return make_pairs(self.date_hn, self.date_dl)

    def descriptions(self):
        return make_pairs(self.description, self.description_en)

    def notes_(self):
        return make_pairs(self.notes, self.notes_en)

    def places(self):
        return make_pairs(self.place_hn, self.place_qn)

    def publishers(self):
        return make_pairs(self.publisher_hn, self.publisher_qn)

    def titles(self):
        return make_pairs(self.title_hn, self.title_qn)

    class Meta(Base.Meta):
        verbose_name_plural = 'metadata'
        ordering = ['nlvnpf_num']

name_re = re.compile("(.*?)(\d+)(.*)")
class Volume(Base):
    """A Volume is a particular bound work.

    A set, such as a reference work, may have multiple volumes.
    """
    def __unicode__(self):
        return "%s-%s" % (self.name, self.md.title_hn)
    nlvnpf = models.CharField(max_length=20)
    collection = models.ForeignKey(Collection, null=True)
    #_set = models.ForeignKey(Set, null=True, blank=True)
    storage = models.ForeignKey(Storage, null=True)
    md = models.ForeignKey(Metadata)
    def sort_key(self):
        """This derived sort order cannot be used in ordering."""
        prefix, numeric, extension = name_re.search(self.name).groups()
        return "%s.%06d%s" % (prefix, int(numeric), extension)

    class Meta(Base.Meta):
        ordering = ['name']


class Page(Base):
    volume = models.ForeignKey(Volume)


class Image(Base):
    page = models.ForeignKey(Page)
